// Copyright (C) 2006 Seb_le_Ouf&Snake.
/*  
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.
   
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
   
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
 */


#ifndef __OBJET__H__
#define __OBJET__H__

#include <iostream>
#include <stdlib.h>
#include <GL/gl.h>
#include <list>

#include <algebra.h>
#include <motor/support.hh>

using namespace std;
using namespace mCG;


/* Constant use for physic calculus */

#define DELTA_T 	1.0/70.0 	// Careful ===> 1./75.
#define G_CONST		9.81
#define	G_VECT		vR3(0., -G_CONST, 0.)


/* --- Class to manage a support type with special features --- */
	
class SPlan
{
	public:
		enum SSide
		{
			S_NO_SIDE,
			S_POS_SIDE,
			S_NEG_SIDE 
		};
		
	protected:
		vR3 sNormal;
		vR3 sDirection;
		double sConstant;
		
	public:
		SPlan();
		SPlan(const vR3& _normal, const double& _constant=0.);
    		SPlan(const vR3& _normal, const pR3& _point);
    		SPlan(const pR3& _point0, const pR3& _point1, const pR3& _point2);
		virtual ~SPlan() {}
		
		void 	setNormal(const vR3&);
		void 	setConstant(const double&);
		void	Normalize();
		
		vR3 	getNormal() const;
		double 	getConstant() const;
		SSide 	getSide(const pR3& _point) const;
		double 	getDistanceTo(const pR3& _p) const;
		pR3	getNearestPoint(const pR3&) const;
};

/* Abstract class for a simple object */

class SInteraction;
class SObject
{
	friend class SInteraction;
	friend class SPerso;
	friend class SMonstre;
	
	protected:
		vR3		sV0;
		pR3 		mass_center;
		double		mass;
		double  	density;
		bool		sVisible;
		bool		sAnimate;
		SSupport*	sSup;
		
		SObject();
		
	public:
		virtual ~SObject() { sSup = NULL;}
		virtual void setMass(const double&);
		virtual void setV0(const vR3&);
		virtual void setDensity(const double&);
		virtual void setVisible(bool b=true);
		virtual void setSupport(SSupport&)=0;
		virtual void setSupport(const SPlan&);
		virtual void setMassCenter(const pR3&);
		virtual void unsetSupport();
		virtual void setAnimated(bool);
		
		virtual double getMass() const;
		virtual double getDensity() const;
		const pR3& getMassCenter() const;
		virtual vR3 getV0() const;
		virtual bool isVisible() const;
		virtual bool isAnimated() const;
		virtual bool hasSupport() const;
		virtual vR3 getSupportDirection() const;
		virtual int getType() const = 0;
		
		virtual void display() const = 0;
		virtual void updatePosition(const pR3&) = 0;
		virtual double getBoundingSphere();
};

class SPoint : public SObject
{
	protected: 
		Color 	sCol;

	public:
		SPoint(double _x=0., double _y=0., double _z=0.);
		SPoint(const double& _x,const double& _y,const double& _z,const Color& _color );
		SPoint(const pR3&);
		
		void setColor(const double&,const double&, const double&);
		void setX(const double& d);
		void setY(const double& d);
		void setZ(const double& d);
		void addX(const double& d);
		void addY(const double& d);
		void addZ(const double& d);
		void setSupport(SSupport&);
		void display() const;
		void updatePosition(const pR3&);
		int getType() const;
};


/* --- A class destinated to build a polyedron figure --- */

class SPolyhedron : public SObject
{
	friend class loaderPLY;
	friend class loaderMAX;
	friend class Model;
	
	public:
		enum DisplayMode
		{
			S_FACE = 1,
			S_WIRE = 2
		};
	
	protected:
		list<SPoint> sVList;
		list<Face>   sFList;
		list<float>  sTList;
		DisplayMode  sMode;
	
	public:
		SPolyhedron(const double& _mass = 0., const pR3& _mCenter = pR3(), const vR3& _v0 = vR3(), const double& _density = 0.);
		SPolyhedron(DisplayMode _mode);
		
		void setDisplayMode(DisplayMode _mode=S_WIRE);
		DisplayMode getDisplayMode() const;
		void setSupport(SSupport&);
		void display() const;
		void updatePosition(const pR3&);
		void add_point(const pR3&);
		int  getType() const;
		
		virtual void rotateX(const double&);
		virtual void rotateY(const double&);
		virtual void rotateZ(const double&);
		
		virtual void applyTransform(const Transform&);
};
			
class SLine : public SPolyhedron
{
	protected:
		pR3 sOrigin;
		vR3 sDirection;
		
	public:
		SLine();
		SLine(const pR3&);
		SLine(const pR3&,const vR3&);
		
		void setOrigin(const pR3&);
		void setDirection(const vR3&);
		
		const pR3& getOrigin() const;
		const vR3& getDirection() const;
		int getType() const;
		
		void display() const;		
};		
	
/* --- A Sphere object with all faces and vetices in memory --- */

class SSphere : public SPolyhedron
{
	protected:
		unsigned slice;
		unsigned stack;
		double rayon;

	public:
	
	SSphere(double, unsigned, unsigned,Color color = Color(1.,1.,1.),const pR3& _c = pR3());
	
	double getRadius() const;
	void setSupport(SSupport&);
	int getType() const;
};	


class SCube : public SPolyhedron
{
	protected:
		double sLong;
		double sHeight;
		double sWidth;
		
	public:
		SCube() {}
		SCube(double _l, double _h, double _w, Color _c=Color(), pR3 _p=pR3());
		const double& getLong() const;
		const double& getHeight() const;
		const double& getWidth() const;
		int getType() const;
};

class SBox : public SCube
{
	protected:
		vR3 sAxis[3];
		double sExtend[3];
		
	public:
		SBox(double _l=0., double _h=0., double _w=0., Color _c=Color(), pR3 _p=pR3()) : SCube(_l,_h,_w,_c,_p)
		{
			sAxis[0] = vR3(1.,0.,0.);
			sAxis[1] = vR3(0.,1.,0.);
			sAxis[2] = vR3(0.,0.,1.);
			
			//mass_center = pR3(_p.x, _p.y - _h, _p.z - _w/2.);
		}
		const vR3& getAxis(int i) const;
		vR3* getAllAxis();
		const vR3* getAllAxis() const;
		const double& getExtend(int i) const;
		double* getAllExtend();
		const double* getAllExtend() const;
		int getType() const;
		
		void rotateX(const double&);
		void rotateY(const double&);
		void rotateZ(const double&);
		void applyTransform(const Transform&);
};


#endif /* __OBJET__H__	*/
